const { MessageEmbed } = require("discord.js");
const prettyMs = require("pretty-ms");
const { sendMessage } = require("@utils/botUtils");

/**
 * Emitted when a track starts.
 * @param {import("@src/structures").BotClient} client
 * @param {import("lavalink-client").Player} player
 * @param {import("lavalink-client").Track} track
 * @param {import("lavalink-client").TrackStartEvent} payload
 */
module.exports = (client, player, track, payload) => {
  const channel = client.channels.cache.get(player.textChannelId);
  const trackInfo = track.info;
  const queueLength = player.queue.tracks.length;

  const embed = new MessageEmbed()
    .setAuthor({ name: "Now Playing" })
    .setColor(client.config.EMBED_COLORS.BOT_EMBED)
    .setDescription(`[${trackInfo.title}](${trackInfo.uri})`)
    .addField("Song Duration", "`" + prettyMs(trackInfo.duration, { colonNotation: true }) + "`", true)
    .setFooter({ text: `Requested By: ${track.requester}` });

  embed.setThumbnail(track.info.artworkUrl);
  if (queueLength) embed.addField("Position in Queue", queueLength.toString(), true);
  sendMessage(channel, { embeds: [embed] });
};